#!/bin/bash
#

decode_length ()
{
	high=0x$7
	mid=0x$8
	low=0x$9
	length=$(( (high << 16) + (mid << 8) + low ))
	echo "length: ($high $mid $low) $length"
}

decode_read ()
{
	dtc=$3
	echo -n "  -> dtc $dtc: "
	case $dtc in
        00)
                echo "Image data" ;;
	60)
		echo "Get calibration format" ;;
        61)
                echo "Read gray calibration data" ;;
        62)
		echo "Read color calibration data" ;;
	64)
		echo "Detect accessories" ;;
	66)
		echo "Read dark calibration data" ;;
	69)
		echo "Read NVM RAM data" ;;
	6a)
		echo "Read Flash RAM information" ;;
	6b)
		echo "Read Flash RAM data" ;;
	6c)
		echo "Read Acceleration Information" ;;
	70)
		echo "Get RAW data format" ;;
	71)
		echo "Get RAW data" ;;
	90)
		echo "Firmware status." ;;
	a0)
		echo "Light source status" ;;
	a1)
		echo "Button status" ;;
	*)
		echo "Error: Unknown!"
	esac
}

decode_send ()
{
	dtc=$3
	echo -n "  -> dtc $dtc: "
	case $dtc in
	6c)
		echo "Download Acceleration Information" ;;
	81)
		echo "Download gamma table" ;;
	82)
		echo "Download calibration data" ;;
	83)
		echo "Download 3x3 color matrix" ;;
	85)
		echo "Download NVRAM data" ;;
	86)
		echo "Download Flash RAM data" ;;
	88)
		echo "Optical unit control" ;;
	90)
		echo "Download firmware" ;;
	92)
		echo "Reset scanner state" ;;
	a0)
		echo "Light source control" ;;
        a2)     
                echo "Power save time" ;;
	*)
		echo "Error: Unknown!"
	esac
}

mark=0

(
while read in ; do
	pattern="*going down*"
	[[ $in == $pattern ]] && mark=1
	pattern="*coming back*"
	[[ $in == $pattern ]] && mark=0
	[ $mark = 1 ] && echo $in
done
) | while read line ; do
  [[ $line == *endpoint* ]] && \
	nep=`echo $line | sed 's/.*\[endpoint \(.*\)\]/\1/'`
  [ "$nep" ] && endpoint="$nep"

  if [[ $line = *0000:* ]] ; then
     cmdtxt="${line/*0000: /}"
     set - $cmdtxt

     txtendpoint=""
     case $endpoint in
	0x*1) txtendpoint="BULK OUT" ;;
	0x*82) txtendpoint="INT IN" ;;
	0x*83) txtendpoint="BULK IN" ;;
     esac

     echo "RAW command: (endpoint: $txtendpoint) $cmdtxt"
     if [[ "$endpoint" = 0x*1 ]] ; then
	echo -n "  $1: "
	case $1 in
	00)
		echo "Test Unit Ready" ;;
	03)
		echo "Request Sense" ;;
	08)
		echo "Check Media" ;;
	12)
		echo "Inquiry" ;;
	16)
		echo "Reserve unit" ;;
	17)
		echo "Release unit" ;;
	1b)
		echo "Scan" ;;
	24)
		echo "Set Window" ;;
	28)
		echo -n "Read "
		decode_length $cmdtxt
		decode_read $cmdtxt
		;;
	2a)
		echo -n "Send "
		decode_length $cmdtxt
		decode_send $cmdtxt
		;;
	31)
		echo "Object position" ;;
	*)
		echo "Error: Unknown command!"
	esac
	echo ""
     else
	echo "non SCSI data ..."
	echo ""
     fi
     endpoint="none"
  fi
done

